announcements = (
    'https://hkrpg-api-static.mihoyo.com'
    '/common/hkrpg_cn/announcement/api/getAnnContent'
    '?game=hkrpg&game_biz=hkrpg_cn&lang=zh-cn&bundle_id=hkrpg_cn'
    '&platform=pc&region=prod_gf_cn&level=70&channel_id=1'
)
